"""JARVIS database models."""
from datetime import datetime, timezone
from functools import partial

from beanie import Document, Link
from pydantic import BaseModel, Field

from jarvis_core.db.models.actions import Ban, Kick, Mute, Unban, Warning
from jarvis_core.db.models.modlog import Action, Modlog, Note
from jarvis_core.db.models.reddit import Subreddit, SubredditFollow
from jarvis_core.db.models.twitter import TwitterAccount, TwitterFollow

__all__ = [
    "Action",
    "Autopurge",
    "Autoreact",
    "Ban",
    "Config",
    "Guess",
    "Kick",
    "Lock",
    "Lockdown",
    "Modlog",
    "Mute",
    "Note",
    "Pin",
    "Pinboard",
    "Purge",
    "Reminder",
    "Rolegiver",
    "Roleping",
    "Setting",
    "Subreddit",
    "SubredditFollow",
    "Temprole",
    "TwitterAccount",
    "TwitterFollow",
    "Unban",
    "UserSetting",
    "Warning",
    "all_models",
]


def get_now() -> datetime:
    """Get proper timestamp."""
    return datetime.now(tz=timezone.utc)


NowField = partial(Field, default_factory=get_now)


class Autopurge(Document):
    guild: int
    channel: int
    delay: int = 30
    admin: int
    created_at: datetime = NowField()


class Autoreact(Document):
    guild: int
    channel: int
    reactions: list[str] = Field(default_factory=list)
    admin: int
    thread: bool
    created_at: datetime = NowField()


class Config(Document):
    """Config database object."""

    key: str
    value: str | int | bool


class Guess(Document):
    """Guess database object."""

    correct: bool
    guess: str
    user: int


class Permission(BaseModel):
    """Embedded Permissions document."""

    id: int
    allow: int = 0
    deny: int = 0


class Lock(Document):
    """Lock database object."""

    active: bool = True
    admin: int
    channel: int
    duration: int = 10
    reason: str
    original_perms: Permission
    created_at: datetime = NowField()


class Lockdown(Document):
    """Lockdown database object."""

    active: bool = True
    admin: int
    duration: int = 10
    guild: int
    reason: str
    original_perms: int
    created_at: datetime = NowField()


class Purge(Document):
    """Purge database object."""

    admin: int
    channel: int
    guild: int
    count: int = 10
    created_at: datetime = NowField()


class Reminder(Document):
    """Reminder database object."""

    active: bool = True
    user: int
    guild: int
    channel: int
    message: str
    remind_at: datetime
    created_at: datetime = NowField()
    private: bool = False


class Rolegiver(Document):
    """Rolegiver database object."""

    guild: int
    roles: list[int]


class Bypass(BaseModel):
    """Roleping bypass embedded object."""

    users: list[int]
    roles: list[int]


class Roleping(Document):
    """Roleping database object."""

    active: bool = True
    role: int
    guild: int
    admin: int
    bypass: Bypass
    created_at: datetime = NowField()


class Setting(Document):
    """Setting database object."""

    guild: int
    setting: str
    value: str | int | bool


class Pinboard(Document):
    """Pinboard database object."""

    channel: int
    guild: int
    admin: int
    created_at: datetime = NowField()


class Pin(Document):
    """Pin database object."""

    active: bool = True
    index: int
    message: int
    channel: int
    pinboard: Link[Pinboard]
    guild: int
    admin: int
    star: int
    created_at: datetime = NowField()


class Temprole(Document):
    """Temporary role object."""

    guild: int
    user: int
    role: int
    admin: int
    expires_at: datetime
    reapply_on_rejoin: bool = True
    created_at: datetime = NowField()


class UserSetting(Document):
    """User Setting object."""

    user: int
    type: str
    settings: str
    value: str | int | bool

    class Setting:
        name = "usersetting"


all_models = [
    Action,
    Autopurge,
    Autoreact,
    Ban,
    Config,
    Guess,
    Kick,
    Lock,
    Lockdown,
    Modlog,
    Mute,
    Note,
    Pin,
    Pinboard,
    Purge,
    Reminder,
    Rolegiver,
    Roleping,
    Setting,
    Subreddit,
    SubredditFollow,
    Temprole,
    TwitterAccount,
    TwitterFollow,
    Unban,
    UserSetting,
    Warning,
]
